
:root {
    --background-url: url('{{ url_for('static', filename='assets/x-square.svg', _external=True) }}');
}

.network-container {
    display: flex;
    align-items: center;
    justify-content: center;
    overflow: auto;
    height: 600px;
    max-height: 600px;
}
.network {
    display: flex;
    flex-direction: column; /* Display layers in a column */
    align-items: center; /* Center layers horizontally within the column */
    height: 600px;
    max-height: 600px;

}


.layer {
    border: 1px solid #ccc;
    padding: 10px;
    margin: 12px;
    border-radius: 15px;
    height:60px;
    width: 700px;
    background-color: aliceblue;
    display: flex;
   align-items: center; /* Center the elements within each layer */

}

.layer-info {
    text-align: center;
    /* margin-bottom: 5px; */
    /* margin-right: 10px; */
    width: 150px;
}

.layer-number {
    font-weight: bold;
}

.layer-dimensions input {
    width: 100px;
    /* margin: 5px; */
    padding: 5px;
    border: 1px solid #ccc;
    border-radius: 3px;
}


.activation-dropdown {
    width: 150px;
    margin-bottom: 6px;
}

.activation-dropdown select {
    width: 100px;
    /* padding: 5px; */
    border: 1px solid #ccc;
    border-radius: 5px;
    margin-left: 5px;
}

.layer-dimensions{
    width: 400px;
}

.layer-type {
    /* font-style: italic; */
    /* margin-top: 10px; */
    display: inline-flex;
    /* margin-left: 10px; */
}


.layer-name {
    /* font-style: italic; */
    margin-top: 10px;
    display: inline-flex;
    /* margin-left: 10px; */
}

.delete-button {
    cursor: pointer;
    height: 20px;
    width: 20px;
    font-size: 20px;
    text-align: center;
    /* 不显示边框和颜色 */
    border: none;
    outline: none;
    background: none;
    padding: 0;
    margin-bottom: 15px;
    margin-right: 15px;
    /* background: var(--background-url) */
    /* background: url("{{ url_for('static',filename='assets/x-square.svg',_external=True) }}") no-repeat 0px 0px; */

}

.layer.deleting {
    opacity: 0;
    transform: scale(0.9);
  }


  #addLayer{
    margin-right: 40px;
  }

  #saveNetwork{
    position: absolute;
    left: 20%;
    top: 30%;
  }

  #network-submit-btn{
    margin-right: 40px;
  }


  #adv-item{
    width: 450px;
  }

  #network-help{
    display: block;
    margin-top:10px;
    margin-left: 40px;
  }